@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2012 Douglas Gilbert.
+ * Copyright (c) 2005-2013 Douglas Gilbert.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -71,13 +71,13 @@ static int map_if_lk24(int sg_fd, const char * device_name, int rw,
 
 #endif  /* SG_LIB_LINUX */
 
-#include "sdparm.h"
 #include "sg_lib.h"
 #include "sg_cmds_basic.h"
+#include "sdparm.h"
 
 #define MAX_DEV_NAMES 256
 
-static char * version_str = "1.07 20120121 [svn: r188]";
+static const char * version_str = "1.08 20130606 [svn: r215]";
 
 
 static struct option long_options[] = {
@@ -158,9 +158,8 @@ usage()
           "    --quiet | -q          suppress device vendor/product/"
           "revision string line\n"
           "    --readonly | -r       force read-only open of DEVICE (def: "
-          "either\n"
-          "                          read-write or read-only). For ATA "
-          "disks\n"
+          "depends\n"
+          "                          on operation). Mainly for ATA disks\n"
           "    --save | -S           place mode changes in saved page as "
           "well\n"
           "    --set=STR | -s STR    set field value(s)\n"
@@ -260,7 +259,7 @@ print_mp_extra(const char * extra)
     char * cp;
     char * p;
 
-    for (p = (char *)extra; (cp = strchr(p, '\t')); p = cp + 1) {
+    for (p = (char *)extra; (cp = (char *)strchr(p, '\t')); p = cp + 1) {
         n = cp - p;
         if (n > (int)(sizeof(b) - 1))
             n = (sizeof(b) - 1);
@@ -373,7 +372,8 @@ list_mp_settings(const struct sdparm_mode_page_settings * mps, int get)
         if (mpip->acron) {
             printf("  acronym: %s", mpip->acron);
             if (mps->it_vals[k].descriptor_num > 0)
-                printf("  descriptor_num=%d\n", mps->it_vals[k].descriptor_num);
+                printf("  descriptor_num=%d\n",
+                       mps->it_vals[k].descriptor_num);
             else
                 printf("\n");
         } else
@@ -468,14 +468,15 @@ print_mp_arr_entry(const char * pre, int smask,
 
 static int
 ll_mode_sense(int fd, const struct sdparm_opt_coll * opts, int pn, int spn,
-              unsigned char * resp, int mx_resp_len, int noisy, int verb)
+              unsigned char * resp, int mx_resp_len, int verb)
 {
     if (opts->mode_6)
         return sg_ll_mode_sense6(fd, opts->dbd, 0 /*current */, pn, spn,
-                                 resp, mx_resp_len, noisy, verb);
+                                 resp, mx_resp_len, 1 /* noisy */, verb);
     else
         return sg_ll_mode_sense10(fd, 0 /* llbaa */, opts->dbd, 0, pn,
-                                  spn, resp, mx_resp_len, noisy, verb);
+                                  spn, resp, mx_resp_len, 1 /* noisy */,
+                                  verb);
 }
 
 /* Make some noise if the mode page response seems badly formed */
@@ -592,7 +593,7 @@ print_direct_access_info(int sg_fd, const struct sdparm_opt_coll * opts,
     unsigned char cur_mp[DEF_MODE_RESP_LEN];
 
     memset(cur_mp, 0, sizeof(cur_mp));
-    res = ll_mode_sense(sg_fd, opts, ALL_MPAGES, 0, cur_mp, 8, 0, verb);
+    res = ll_mode_sense(sg_fd, opts, ALL_MPAGES, 0, cur_mp, 8, verb);
     if (0 == res) {
         v = cur_mp[opts->mode_6 ? 2 : 3];
         printf("    Direct access device specific parameters: WP=%d  "
@@ -1183,7 +1184,7 @@ change_mode_page(int sg_fd, int pdt,
     mpp = sdp_get_mpage_name(pn, spn, pdt, opts->transport, opts->vendor,
                              0, 0, b, sizeof(b));
     memset(mdpg, 0, sizeof(mdpg));
-    res = ll_mode_sense(sg_fd, opts, pn, spn, mdpg, 4, 1, opts->verbose);
+    res = ll_mode_sense(sg_fd, opts, pn, spn, mdpg, 4, opts->verbose);
     if (0 != res) {
         if (SG_LIB_CAT_NOT_READY == res)
             fprintf(stderr, "mode sense command failed, device not ready\n");
@@ -1208,7 +1209,7 @@ change_mode_page(int sg_fd, int pdt,
                 "allocation length=%d\n", md_len, (int)sizeof(mdpg));
         return SG_LIB_CAT_MALFORMED;
     }
-    res = ll_mode_sense(sg_fd, opts, pn, spn, mdpg, md_len, 1, opts->verbose);
+    res = ll_mode_sense(sg_fd, opts, pn, spn, mdpg, md_len, opts->verbose);
     if (0 != res) {
         fprintf(stderr, "change_mode_page: failed fetching page: %s\n", b);
         return res;
@@ -1319,7 +1320,7 @@ set_def_mode_page(int sg_fd, int pn, int spn, unsigned char * mode_pg,
         return SG_LIB_FILE_ERROR;
     }
     memset(mdp, 0, len);
-    ret = ll_mode_sense(sg_fd, opts, pn, spn, mdp, 4, 1, opts->verbose);
+    ret = ll_mode_sense(sg_fd, opts, pn, spn, mdp, 4, opts->verbose);
     if (0 != ret) {
         sdp_get_mpage_name(pn, spn, -1, opts->transport, opts->vendor, 0, 0,
                            buff, sizeof(buff));
@@ -1334,7 +1335,7 @@ set_def_mode_page(int sg_fd, int pn, int spn, unsigned char * mode_pg,
         ret = SG_LIB_CAT_MALFORMED;
         goto err_out;
     }
-    ret = ll_mode_sense(sg_fd, opts, pn, spn, mdp, md_len, 1, opts->verbose);
+    ret = ll_mode_sense(sg_fd, opts, pn, spn, mdp, md_len, opts->verbose);
     if (0 != ret) {
         sdp_get_mpage_name(pn, spn, -1, opts->transport, opts->vendor,
                            0, 0, buff, sizeof(buff));
@@ -1725,7 +1726,7 @@ open_and_simple_inquiry(const char * device_name, int rw, int * pdt,
     char b[32];
 
     verb = (opts->verbose > 0) ? opts->verbose - 1 : 0;
-    sg_fd = sg_cmds_open_device(device_name, ! rw, verb);
+    sg_fd = sg_cmds_open_device(device_name, ! rw /* read_only */ , verb);
     if (sg_fd < 0) {
         fprintf(stderr, "open error: %s [%s]: %s\n", device_name,
                 (rw ? "read/write" : "read only"), safe_strerror(-sg_fd));
@@ -1777,8 +1778,8 @@ err_out:
 /* Process mode page(s) operation. Returns 0 if successful */
 static int
 process_mode(int sg_fd, const struct sdparm_mode_page_settings * mps, int pn,
-             int spn, int rw, int get, const struct sdparm_opt_coll * opts,
-             int pdt)
+             int spn, int set_clear, int get,
+             const struct sdparm_opt_coll * opts, int pdt)
 {
     int res;
     const struct sdparm_mode_page_t * mpp;
@@ -1811,7 +1812,7 @@ process_mode(int sg_fd, const struct sdparm_mode_page_settings * mps, int pn,
     }
     if (opts->defaults)
         res = set_mp_defaults(sg_fd, pn, spn, pdt, opts);
-    else if (rw) {
+    else if (set_clear) {
         if (mps->num_it_vals < 1) {
             fprintf(stderr, "no fields found to set or clear\n");
             return SG_LIB_CAT_OTHER;
@@ -1845,7 +1846,8 @@ main(int argc, char * argv[])
     int num_devices = 0;
     int pn = -1;
     int spn = -1;
-    int rw = 0;
+    int rw = 0;         /* 1: requires RDWR, 0: perhaps RDONLY ok */
+    int set_clear = 0;
     int protect = 0;
     int cmd_arg = -1;
     const struct sdparm_mode_page_t * mpp = NULL;
@@ -1892,6 +1894,7 @@ main(int argc, char * argv[])
             break;
         case 'c':
             clear_str = optarg;
+            set_clear = 1;
             rw = 1;
             break;
         case 'C':
@@ -1971,6 +1974,7 @@ main(int argc, char * argv[])
         case 's':
             set_str = optarg;
             rw = 1;
+            set_clear = 1;
             break;
         case 'S':
             ++opts.save;
@@ -2102,7 +2106,7 @@ main(int argc, char * argv[])
                 pdt = mpp->pdt;
             }
         } else {        /* got page_str and first char probably numeric */
-            cp = strchr(page_str, ',');
+            cp = (char *)strchr(page_str, ',');
             pn = sg_get_num_nomult(page_str);
             if ((pn < 0) || (pn > 255)) {
                 fprintf(stderr, "Bad page code value after '-p' "
@@ -2136,8 +2140,7 @@ main(int argc, char * argv[])
     }
 
     if (opts.inquiry) {
-        if (set_str || clear_str || get_str || cmd_str || opts.defaults ||
-            opts.save) {
+        if (set_clear || get_str || cmd_str || opts.defaults || opts.save) {
             fprintf(stderr, "'--inquiry' option lists VPD pages so other "
                     "options that are\nconcerned with mode pages are "
                     "inappropriate\n");
@@ -2153,7 +2156,7 @@ main(int argc, char * argv[])
             return 0;
         }
     } else if (cmd_str) {
-        if (set_str || clear_str || get_str || opts.defaults || opts.save) {
+        if (set_clear || get_str || opts.defaults || opts.save) {
             fprintf(stderr, "'--command=' option is not valid with other "
                     "options that are\nconcerned with mode pages\n");
             return SG_LIB_SYNTAX_ERROR;
@@ -2182,7 +2185,7 @@ main(int argc, char * argv[])
             mp_settings.subpage_num = spn;
         }
         if (get_str) {
-            if (set_str || clear_str) {
+            if (set_clear) {
                 fprintf(stderr, "'--get=' can't be used with '--set=' "
                         "or " "'--clear='\n");
                 return SG_LIB_SYNTAX_ERROR;
@@ -2191,8 +2194,7 @@ main(int argc, char * argv[])
                 return SG_LIB_SYNTAX_ERROR;
         }
         if (1 == opts.enumerate) {
-            if ((num_devices > 0) || set_str || clear_str || get_str ||
-                opts.save)
+            if ((num_devices > 0) || set_clear || get_str || opts.save)
                 /* think about --get= with --enumerate */
                 printf("<scsi_device> as well as most options are ignored "
                        "when '--enumerate' is given\n");
@@ -2303,7 +2305,7 @@ main(int argc, char * argv[])
             return SG_LIB_SYNTAX_ERROR;
         }
 
-        if (opts.defaults && (set_str || clear_str || get_str)) {
+        if (opts.defaults && (set_clear || get_str)) {
             fprintf(stderr, "'--get=', '--set=' or '--clear=' "
                     "can't be used with '--defaults'\n");
             return SG_LIB_SYNTAX_ERROR;
@@ -2358,7 +2360,7 @@ main(int argc, char * argv[])
             if (cmd_str && scmdp)   /* process command */
                 r = sdp_process_cmd(sg_fd, scmdp, cmd_arg, pdt, &opts);
             else                    /* mode page */
-                r = process_mode(sg_fd, &mp_settings, pn, spn, rw,
+                r = process_mode(sg_fd, &mp_settings, pn, spn, set_clear,
                                  (NULL != get_str), &opts, pdt);
         }
 
