@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005-2011 Douglas Gilbert.
+ * Copyright (c) 2005-2013 Douglas Gilbert.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -28,10 +28,13 @@
  */
 
 #include <stdlib.h>
-#include "sdparm.h"
 #include "sg_lib.h"
+#include "sdparm.h"
 
 
+#define PROTO_IDENT_STR "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas/spl; " \
+        "7: adt;\t8: ata/acs; 9: uas; 10: sop"
+
 /*
  * sdparm is a utility program to access and change SCSI device
  * (logical unit) mode page fields and do some other housekeeping.
@@ -132,6 +135,7 @@ struct sdparm_mode_page_t sdparm_gen_mode_pg[] = {
         NULL},
     {WRITE_PARAM_MP, 0, PDT_MMC, 0, "wp", "Write parameters (MMC)", NULL},
     {XOR_MP, 0, PDT_DISK, 0, "xo", "XOR control (SBC)", NULL},
+        /* XOR control mode page made obsolete in sbc3r32 */
     {0, 0, 0, 0, NULL, NULL, NULL},
 };
 
@@ -145,13 +149,12 @@ struct sdparm_transport_id_t sdparm_transport_id[] = {
     {TPROTO_1394, "sbp", "Serial bus (SBP)"}, /* none */
     {TPROTO_SRP, "srp", "SCSI remote DMA (SRP)"},
     {TPROTO_ISCSI, "iscsi", "Internet SCSI (iSCSI)"}, /* none */
-    {TPROTO_SAS, "sas", "Serial attached SCSI (SAS)"},
+    {TPROTO_SAS, "sas", "Serial attached SCSI (SAS, SPL) "},
     {TPROTO_ADT, "adt", "Automation/Drive interface (ADT)"},
-    {TPROTO_ATA, "ata", "AT attachment interface (ATA/ATAPI)"},
-                                                         /* none */
-    {0x9, "u0x9", NULL},      /* leading "u" so not number */
-    {0xa, "u0xa", NULL},
-    {0xb, "u0xb", NULL},
+    {TPROTO_ATA, "ata", "AT attachment interface (ATA, ACS)"}, /* none */
+    {TPROTO_UAS, "uas", "USB attached SCSI (UAS)"}, /* none */
+    {TPROTO_SOP, "sop", "SCSI over PCIe (SOP)"}, /* none */
+    {0x9, "u0xb", NULL},      /* leading "u" so not number */
     {0xc, "u0xc", NULL},
     {0xd, "u0xd", NULL},
     {0xe, "u0xe", NULL},
@@ -192,18 +195,18 @@ static struct sdparm_mode_page_t sdparm_srp_mode_pg[] = {    /* SRP */
     {0, 0, 0, 0, NULL, NULL, NULL},
 };
 
-static struct sdparm_mode_descriptor_t sas_pcd_desc = {   /* desc SAS-2 */
+static struct sdparm_mode_descriptor_t sas_pcd_desc = {   /* desc SAS/SPL */
     7, 1, 0, 8, 48, -1, -1, "SAS phy"
 };
 
-static struct sdparm_mode_descriptor_t sas_e_phy_desc = {  /* desc SAS-2 */
+static struct sdparm_mode_descriptor_t sas_e_phy_desc = {  /* desc SAS/SPL */
     7, 1, 0, 8, -1, 2, 2, "Enhanced phy"
 };
 
 /* N.B. In SAS 2.1 the spec was split with the upper levels going into the
  * SAS Protocol Layer (SPL) document. So now the SPL drafts are the
  * relevant SAS references. */
-static struct sdparm_mode_page_t sdparm_sas_mode_pg[] = {    /* SAS-2 */
+static struct sdparm_mode_page_t sdparm_sas_mode_pg[] = {    /* SAS/SPL */
     {DISCONNECT_MP, 0, -1, 0, "dr", "Disconnect-reconnect (SAS)", NULL},
     {PROT_SPEC_LU_MP, 0, -1, 0, "pl", "Protocol specific logical unit (SAS)",
         NULL},
@@ -268,6 +271,7 @@ struct sdparm_vpd_page_t sdparm_vpd_pg[] = {
     {VPD_SCSI_PORTS, 0, -1, "sp", "SCSI ports"},
     {VPD_SUPPORTED_VPDS, 0, -1, "sv", "Supported VPD pages"},
     {VPD_TA_SUPPORTED, 0, PDT_TAPE, "tas", "TapeAlert supported flags (SSC)"},
+    {VPD_3PARTY_COPY, 0, -1, "tpc", "Third party copy"},
     {0, 0, 0, NULL, NULL},
 };
 
@@ -515,6 +519,11 @@ struct sdparm_mode_page_item sdparm_mitem_arr[] = {
         "0: CSS unit is bytes; 1: CSS unit is blocks"},
     {"DRA", CACHING_MP, 0, PDT_DISK, 12, 5, 1, 0,
         "Disable read ahead", NULL},
+    {"SYNC_PROG", CACHING_MP, 0, PDT_DISK, 12, 2, 2, 0, /* sbc3r33 */
+        "Synchronous cache progress indication",
+        "0: no pollable sense data during sync\t"
+        "1: allow pollable sense data, allow all commands during sync\t"
+        "2: allow pollable sense data, allow some commands during sync"},
     {"NV_DIS", CACHING_MP, 0, PDT_DISK, 12, 0, 1, 0,
         "Non-volatile cache disable", NULL},
     {"NCS", CACHING_MP, 0, PDT_DISK, 13, 7, 8, 0,
@@ -588,6 +597,8 @@ struct sdparm_mode_page_item sdparm_mitem_arr[] = {
     {"INIT_PR", CONTROL_MP, MSP_SPC_CE, -1, 5, 3, 4, 0,
         "Initial command priority", "0: none or vendor\t"
         "1: highest\t15: lowest"},
+    {"MSDL", CONTROL_MP, MSP_SPC_CE, -1, 6, 7, 8, 0,  /* spc4r34 */
+        "Maximum sense data length", "0: unlimited"},
 
     /* Application tag mode subpage [0xa,0xf0] sbc3 */
     /* descriptor starts here, <start_byte> is relative to start of mode
@@ -668,7 +679,7 @@ struct sdparm_mode_page_item sdparm_mitem_arr[] = {
         "Decompression algorithm",
         "0: none; 5: ALDC (2048 byte): 16: IDRC; 32: DCLZ"},
 
-    /* XOR control mode page [0x10] sbc2 */
+    /* XOR control mode page [0x10] sbc2 << obsolete in sbc3r32>> */
     {"XORDIS", XOR_MP, 0, PDT_DISK, 2, 1, 1, 0,
         "XOR disable", NULL},
     {"MXWS", XOR_MP, 0, PDT_DISK, 4, 7, 32, 0,
@@ -787,13 +798,13 @@ struct sdparm_mode_page_item sdparm_mitem_arr[] = {
     /* Protocol specific logical unit control mode page [0x18] spc3 */
     {"LUPID", PROT_SPEC_LU_MP, 0, -1, 2, 3, 4, 0,
         "Logical unit's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi\t"
+        PROTO_IDENT_STR "\t"
         "[try adding '-t <transport>' to get more fields]"},
 
     /* Protocol specific port control mode page [0x19] spc3 */
     {"PPID", PROT_SPEC_PORT_MP, 0, -1, 2, 3, 4, 0,
         "Port's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi\t"
+        PROTO_IDENT_STR "\t"
         "[try adding '-t <transport>' to get more fields]"},
 
     /* Power condition mode page [0x1a] spc3 (expanded in spc4r18) */
@@ -825,15 +836,15 @@ struct sdparm_mode_page_item sdparm_mitem_arr[] = {
         "Idle_c condition timer (100 ms)", NULL},
     {"SYCT", POWER_MP, 0, -1, 20, 7, 32, 0,
         "Standby_y condition timer (100 ms)", NULL},
-    {"FIDCPC", POWER_MP, 0, -1, 39, 7, 2, 0,    /* added spc4r25 */
-        "From idle command processing control",
-        "0: reserved (SAS-2); 1: disabled; 2: enabled\n"},
-    {"FSBCPC", POWER_MP, 0, -1, 39, 5, 2, 0,    /* added spc4r25 */
-        "From standby command processing control",
-        "0: reserved (SAS-2); 1: disabled; 2: enabled\n"},
-    {"FSTCPC", POWER_MP, 0, -1, 39, 3, 2, 0,    /* added spc4r25 */
-        "From stopped command processing control",
-        "0: reserved (SAS-2); 1: disabled; 2: enabled\n"},
+    {"CCF_IDLE", POWER_MP, 0, -1, 39, 7, 2, 0,     /* changed spc4r35 */
+        "check condition on transition from idle", /* was FIDCPC (spc4r25) */
+        "0: restricted (SAS-2); 1: disabled; 2: enabled\n"},
+    {"CCF_STAND", POWER_MP, 0, -1, 39, 5, 2, 0,        /* changed spc4r35 */
+        "check condition on transition from standby",  /* was FSBCPC */
+        "0: restricted (SAS-2); 1: disabled; 2: enabled\n"},
+    {"CCF_STOPP", POWER_MP, 0, -1, 39, 3, 2, 0,        /* changed spc4r35 */
+        "check condition on transition from stopped",  /* was FSTCPC */
+        "0: restricted (SAS-2); 1: disabled; 2: enabled\n"},
 
     /* Power consumption mode page [0x1a,1] added spc4r33 */
     {"ps_id", POWER_MP, MSP_SPC_PS, -1, 7, 7, 8, 0,      /* added spc4r24 */
@@ -1227,14 +1238,14 @@ static struct sdparm_mode_page_item sdparm_mitem_fcp_arr[] = {
     /* protocol specific logical unit mode page [0x18] fcp3 */
     {"LUPID", PROT_SPEC_LU_MP, 0, -1, 2, 3, 4, MF_COMMON | MF_CLASH_OK,
         "Logical unit's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi"},
+        PROTO_IDENT_STR },
     {"EPDC", PROT_SPEC_LU_MP, 0, -1, 3, 0, 1, MF_COMMON,
         "Enable precise delivery checking", NULL},
 
     /* protocol specific port control page [0x19] fcp3 */
     {"PPID", PROT_SPEC_PORT_MP, 0, -1, 2, 3, 4, MF_COMMON | MF_CLASH_OK,
         "Port's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi"},
+        PROTO_IDENT_STR },
     {"DTFD", PROT_SPEC_PORT_MP, 0, -1, 3, 7, 1, MF_COMMON,
         "Disable target fabric discovery", NULL},
     {"PLPB", PROT_SPEC_PORT_MP, 0, -1, 3, 6, 1, MF_COMMON,
@@ -1285,19 +1296,19 @@ static struct sdparm_mode_page_item sdparm_mitem_spi_arr[] = {
     /* protocol specific logical unit control mode page [0x18] spi4 */
     {"LUPID", PROT_SPEC_LU_MP, 0, -1, 2, 3, 4, MF_COMMON | MF_CLASH_OK,
         "Logical unit's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi"},
+        PROTO_IDENT_STR },
 
     /* protocol specific port control page [0x19] spi4 */
     {"PPID", PROT_SPEC_PORT_MP, 0, -1, 2, 3, 4, MF_COMMON | MF_CLASH_OK,
         "Port's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi"},
+        PROTO_IDENT_STR },
     {"STT", PROT_SPEC_PORT_MP, 0, -1, 4, 7, 16, MF_COMMON,
         "Synchronous transfer timeout (ms)", NULL},
 
     /* margin control subpage  [0x19,0x1] spi4 */
     {"PPID_1", PROT_SPEC_PORT_MP, MSP_SPI_MC, -1, 5, 3, 4, 0,
         "Port's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi"},
+        PROTO_IDENT_STR },
     {"DS", PROT_SPEC_PORT_MP, MSP_SPI_MC, -1, 7, 7, 4, 0,
         "Driver strength", NULL},
     {"DA", PROT_SPEC_PORT_MP, MSP_SPI_MC, -1, 8, 7, 4, 0,
@@ -1310,7 +1321,7 @@ static struct sdparm_mode_page_item sdparm_mitem_spi_arr[] = {
     /* saved training configuration subpage [0x19,0x2] spi4 */
     {"PPID_2", PROT_SPEC_PORT_MP, MSP_SPI_STC, -1, 5, 3, 4, 0,
         "Port's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi"},
+        PROTO_IDENT_STR },
     {"DB0", PROT_SPEC_PORT_MP, MSP_SPI_STC, -1, 10, 7, 32, MF_HEX,
         "DB(0) value", NULL},
     {"DB1", PROT_SPEC_PORT_MP, MSP_SPI_STC, -1, 14, 7, 32, MF_HEX,
@@ -1369,7 +1380,7 @@ static struct sdparm_mode_page_item sdparm_mitem_spi_arr[] = {
     /* negotiated settings subpage [0x19,0x3] spi4 */
     {"PPID_3", PROT_SPEC_PORT_MP, MSP_SPI_NS, -1, 5, 3, 4, 0,
         "Port's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi"},
+        PROTO_IDENT_STR },
     {"TPF", PROT_SPEC_PORT_MP, MSP_SPI_NS, -1, 6, 7, 8, 0,
         "Transfer period factor", NULL},
     {"RAO", PROT_SPEC_PORT_MP, MSP_SPI_NS, -1, 8, 7, 8, 0,
@@ -1388,7 +1399,7 @@ static struct sdparm_mode_page_item sdparm_mitem_spi_arr[] = {
     /* report transfer capabilities subpage [0x19,0x4] spi4 */
     {"PPID_4", PROT_SPEC_PORT_MP, MSP_SPI_RTC, -1, 5, 3, 4, 0,
         "Port's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi"},
+        PROTO_IDENT_STR },
     {"MTPF", PROT_SPEC_PORT_MP, MSP_SPI_RTC, -1, 6, 7, 8, 0,
         "Minimum transfer period factor", NULL},
     {"MRAO", PROT_SPEC_PORT_MP, MSP_SPI_RTC, -1, 8, 7, 8, 0,
@@ -1414,27 +1425,28 @@ static struct sdparm_mode_page_item sdparm_mitem_srp_arr[] = {
 };
 
 static struct sdparm_mode_page_item sdparm_mitem_sas_arr[] = {
-    /* disconnect-reconnect mode page [0x2] sas1 */
+    /* disconnect-reconnect mode page [0x2] sas/spl */
     {"BITL", DISCONNECT_MP, 0, -1, 4, 7, 16, MF_COMMON,
         "Bus inactivity time limit (100us)", NULL},
     {"MCTL", DISCONNECT_MP, 0, -1, 8, 7, 16, MF_COMMON,
         "Maximum connect time limit (100us)", NULL},
     {"MBS", DISCONNECT_MP, 0, -1, 10, 7, 16, MF_COMMON | MF_CLASH_OK,
         "Maximum burst size (512 bytes)", NULL},
+        /* obsoleted spl3r2, re-instated spl3r3 */
     {"FBS", DISCONNECT_MP, 0, -1, 14, 7, 16, MF_CLASH_OK,
         "First burst size (512 bytes)", NULL},
 
-    /* protocol specific logical unit mode page [0x18] sas2 */
+    /* protocol specific logical unit mode page [0x18] sas/spl */
     {"LUPID", PROT_SPEC_LU_MP, 0, -1, 2, 3, 4, MF_COMMON | MF_CLASH_OK,
         "Logical unit's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi"},
+        PROTO_IDENT_STR },
     {"TLR", PROT_SPEC_LU_MP, 0, -1, 2, 4, 1, 0,
         "Transport layer retries (supported)", NULL},
 
-    /* protocol specific port mode page [0x19] sas2 */
+    /* protocol specific port mode page [0x19] sas/spl */
     {"PPID", PROT_SPEC_PORT_MP, 0, -1, 2, 3, 4, MF_COMMON | MF_CLASH_OK,
         "Port's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi"},
+        PROTO_IDENT_STR },
     {"CAWT", PROT_SPEC_PORT_MP, 0, -1, 2, 6, 1, 0,
         "Continue AWT (arbitration wait time (timer))", NULL},
     {"BAE", PROT_SPEC_PORT_MP, 0, -1, 2, 5, 1, 0,
@@ -1454,10 +1466,10 @@ static struct sdparm_mode_page_item sdparm_mitem_sas_arr[] = {
         "Reject to open limit (10 us)",         /* added in sas2r14 */
         "0: vendor specific"},
 
-    /* phy control and discover mode page [0x19,0x1] sas2 */
+    /* phy control and discover mode page [0x19,0x1] sas/spl */
     {"PPID_1", PROT_SPEC_PORT_MP, MSP_SAS_PCD, -1, 5, 3, 4, 0,
         "Port's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi"},
+        PROTO_IDENT_STR },
     {"GENC", PROT_SPEC_PORT_MP, MSP_SAS_PCD, -1, 6, 7, 8, 0,
         "Generation code", "0: unknown, 1..255: valid"},
     {"NOP", PROT_SPEC_PORT_MP, MSP_SAS_PCD, -1, 7, 7, 8, MF_COMMON,
@@ -1479,10 +1491,11 @@ static struct sdparm_mode_page_item sdparm_mitem_sas_arr[] = {
         "Reason (for starting link reset)",
         "0: unknown; 1: power on; 2: hard reset; 3: SMP phy control\t"
         "4: loss of dword sync; 5: mux problem; ..."},
-    {"NPLR", PROT_SPEC_PORT_MP, MSP_SAS_PCD, -1, 13, 3, 4, 0,
+    {"NLLR", PROT_SPEC_PORT_MP, MSP_SAS_PCD, -1, 13, 3, 4, 0,
         "Negotiated logical link rate",         /* sas2r07 */
         "0: unknown; 1: disabled; 2: phy reset problem; 3: spinup hold\t"
-        "4: port selector; 8: 1.5 Gbps; 9: 3 Gbps; 10: 6 Gbps; 11: 12 Gbps"},
+        "4: port selector; 5: resetting; 6: attached unsupported\t"
+        "8: 1.5 Gbps; 9: 3 Gbps; 10: 6 Gbps; 11: 12 Gbps"},
     {"ASIP", PROT_SPEC_PORT_MP, MSP_SAS_PCD, -1, 14, 3, 1, 0,
         "Attached SSP initiator port", NULL},
     {"ATIP", PROT_SPEC_PORT_MP, MSP_SAS_PCD, -1, 14, 2, 1, 0,
@@ -1526,19 +1539,19 @@ static struct sdparm_mode_page_item sdparm_mitem_sas_arr[] = {
         "Hardware maximum link rate",
         "8: 1.5 Gbps; 9: 3 Gbps; 10: 6 Gbps; 11: 12 Gbps"},
 
-    /* shared port control mode page [0x19,0x2] sas2 */
+    /* shared port control mode page [0x19,0x2] sas/spl */
     {"PPID_2", PROT_SPEC_PORT_MP, MSP_SAS_SPC, -1, 5, 3, 4, 0,
         "Port's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi"},
+        PROTO_IDENT_STR },
     {"PLT", PROT_SPEC_PORT_MP, MSP_SAS_SPC, -1, 6, 7, 16, 0,
         "Power loss timeout(ms)", NULL},
     {"PGRATO", PROT_SPEC_PORT_MP, MSP_SAS_SPC, -1, 9, 7, 8, 0,
         "Power grant timeout(sec)", NULL},
 
-    /* SAS-2 Enhanced phy mode page [0x19,0x3] sas2, spl */
+    /* SAS-2 Enhanced phy mode page [0x19,0x3] sas/spl */
     {"PPID_3", PROT_SPEC_PORT_MP, MSP_SAS_E_PHY, -1, 5, 3, 4, 0,
         "Port's (transport) protocol identifier",
-        "0: fcp; 1: spi; 4: srp; 5: iscsi; 6: sas; 7: adt; 8: ata/atapi"},
+        PROTO_IDENT_STR },
     {"GENC_1", PROT_SPEC_PORT_MP, MSP_SAS_E_PHY, -1, 6, 7, 8, 0,
         "Generation code", "0: unknown, 1..255: valid"},
     {"NOP_1", PROT_SPEC_PORT_MP, MSP_SAS_E_PHY, -1, 7, 7, 8, 0,
@@ -1557,9 +1570,10 @@ static struct sdparm_mode_page_item sdparm_mitem_sas_arr[] = {
         "Optical mode enabled", NULL},
     {"N_SSC", PROT_SPEC_PORT_MP, MSP_SAS_E_PHY, -1, 26, 4, 1, 0,
         "Negotiated spread spectrum clocking", NULL},
-    {"N_PLR", PROT_SPEC_PORT_MP, MSP_SAS_E_PHY, -1, 26, 3, 4, 0,
+    {"NPLR", PROT_SPEC_PORT_MP, MSP_SAS_E_PHY, -1, 26, 3, 4, 0,
         "Negotiated physical link rate",
-        "6: resetting; 7: attached unsupported\t"
+        "0: unknown; 1: disabled; 2: phy reset problem; 3: spinup hold\t"
+        "4: port selector; 5: resetting; 6: attached unsupported\t"
         "8: 1.5 Gbps; 9: 3 Gbps; 10: 6 Gbps; 11: 12 Gbps"},
     {"EN_SL", PROT_SPEC_PORT_MP, MSP_SAS_E_PHY, -1, 27, 2, 1, 0,
         "Enable slumber phy power condition", NULL},
@@ -1651,7 +1665,8 @@ const char * sdparm_desig_type_arr[] =
     "Logical unit group",
     "MD5 logical unit identifier",
     "SCSI name string",
-    "Reserved [0x9]", "Reserved [0xa]", "Reserved [0xb]",
+    "Protocol specific port identifier",        /* spc4r36 */
+    "Reserved [0xa]", "Reserved [0xb]",
     "Reserved [0xc]", "Reserved [0xd]", "Reserved [0xe]", "Reserved [0xf]",
 };
 
